

class FlyerFactory {

	public static function createFlyer(type : String, x : Float, y : Float, radian : Float) : FlyingObject {
		var flyer : FlyingObject;

		switch(type) {
			case "yellow":
				flyer = new ShipBadYellow(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorSinusoid(flyer, new OutOfScreenBehaviorHCylinder(flyer), 0.5);
				flyer.addWeapon(new WeaponGuide("plasma", flyer, 2000, 7));
				flyer.speed = 4;
				flyer.life = 2;
				flyer.fireRate = 0.005;

			case "yellow2":
				flyer = new ShipBadYellow(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorBigOSin(flyer);
				flyer.addWeapon(new WeaponGuide("plasma", flyer, 2000, 7));
				flyer.speed = 4;
				flyer.life = 2;
				flyer.fireRate = 0.005;

			case "green":
				flyer = new ShipBadGreen(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorZ(flyer, new OutOfScreenBehaviorHCylinder(flyer));
				flyer.addWeapon(new WeaponBomb("redlaser", flyer, 500));
				flyer.speed = 4;
				flyer.life = 3;
				flyer.fireRate = 0.10;

			case "red":
				flyer = new ShipBadRed(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorU(flyer);
				flyer.addWeapon(new WeaponGuide("redcircle", flyer, 2000));
				flyer.speed = 4;
				flyer.life = 2;

			case "star":
				flyer = new ShipBadStarRed(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorZ(flyer, new OutOfScreenBehaviorHCylinder(flyer));
				flyer.addWeapon(new WeaponGuide6("redcircle", flyer, 3000));
				flyer.rotationSpeed = 0.05;
				flyer.speed = 3;
				flyer.life = 4;

			case "shuriken":
				flyer = new ShipBadShuriken(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorShuriken(flyer);
				flyer.rotationSpeed = 0.10;
				flyer.speed = 3;
				flyer.life = 4;

			case "redmine":
				flyer = new ProjectileRedMine(x, y, radian, 0); 
				flyer.life = 10;
				flyer.speed = 1;
				flyer.rotationSpeed = 0.06;

			case "magneticmine":
				flyer = new ShipBadMagneticMine(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorGuided(flyer, new OutOfScreenBehaviorRemove(flyer));
				flyer.rotationSpeed = 0.07;
				flyer.speed = 3;
				flyer.life = 4;

			case "bluebomb":
				flyer = new ShipBadBlueBomb(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorSinusoid(flyer, new OutOfScreenBehaviorHCylinder(flyer), 0.3);
				flyer.rotationSpeed = 0.07;
				flyer.speed = 2.5;
				flyer.life = 6;
				flyer.fireRate = 1;

			case "asteroid":
				flyer = new ShipBadAsteroid(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorStraight(flyer, new OutOfScreenBehaviorRemove(flyer));
				flyer.rotationSpeed = 0.02 * (1 + Std.random(5));
				flyer.scale(Math.random() / 3 + 0.5);
				flyer.life = Math.round(flyer.scaleX * 8);
				//flyer.speed = Math.round(flyer.scaleX * 8);
				flyer.speed = flyer.scaleX * 8;

			case "smallboss":
				flyer = new ShipBadSmallBoss(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorT(flyer, new OutOfScreenBehaviorClosedBox(flyer));
				flyer.addWeapon(new WeaponTriple("plasma", flyer, 500));
				flyer.addWeapon(new WeaponGuide("redcircle", flyer, 1000));
				flyer.speed = 2;
				flyer.life = 20;

			case "bonusred":
				flyer = new BonusRed(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorStraight(flyer, new OutOfScreenBehaviorRemove(flyer));
				flyer.speed = 3;

			case "bonusyellow":
				flyer = new BonusYellow(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorSinusoid(flyer, new OutOfScreenBehaviorRemove(flyer), 0.8);
				flyer.speed = 3;

			case "bonuslife":
				flyer = new BonusLife(x, y, radian);
				flyer.flyBehavior = new FlyBehaviorStraight(flyer, new OutOfScreenBehaviorRemove(flyer));
				flyer.speed = 3;

			default:
				trace("ERR: unknown flyer '" + type + "'");
				flyer = null;
		}

		return flyer;
	}
}
