import flash.display.MovieClip;
import flash.display.Sprite;
import flash.geom.ColorTransform;
import flash.events.TimerEvent;
import flash.utils.Timer;


class FlyingObject extends MovieClip {
	// Etat du vaisseau
	public var life : Int;
	public var state : String;

	// Largeur / hauteur du bitmapdata associe au sprite
	public var bdWidth : Float;
	public var bdHeight : Float;

	// Coordonnees du centre
	public var centerX : Float;	
	public var centerY : Float;

	// Angle et vitesse de deplacement
	public var radian : Float; // angle de deplacement
	public var speed : Float; // vitesse de deplacement
	public var rotationRad : Float;  // angle de rotation du sprite
	public var rotationSpeed : Float;  // vitesse de rotation

	// Permettent de retrouver (x,y) a partir de (centerX,centerY)
	public var angle : Float;	
	public var radius : Float;

	// Modalite de deplacement
	public var flyBehavior : FlyBehavior;

	// Armes / equipement
	var weapon : Array<Weapon>;
	public var fireRate : Float;

	// Liste des cibles
	public var target : Array<FlyingObject>;

	// Permet de gerer le temps d'explosion
	var exploseOn : Bool;
	var exploseTimer : Timer;

	// Effets speciaux sur la couleur
	var blinkOn : Bool;
	var blinkTimer : Timer;
	var colorAngle : Float;
	var colorAngleSpeed : Float;
	var colorFx : ColorTransform;


	public function new(?cx : Float, ?cy : Float, ?radian : Float) {
		super();

		bdHeight = height;
		bdWidth = width;
		angle = - Math.atan2(bdWidth, bdHeight);
		radius = Math.sqrt(bdHeight * bdHeight + bdWidth * bdWidth) / 2;

		weapon = new Array();
		fireRate = 0.01;
		life = 0;

		state = "ok";
		exploseOn = false;
		blinkOn = false;

		colorFx = new ColorTransform(1, 1, 1, 1, 0, 0, 0, 0);

		this.radian = (radian != null) ? radian : 0;
		rotationRad = radian;
		rotation = 180 * rotationRad / Math.PI;

		speed = 0;
		rotationSpeed = 0;

		centerX = (cx != null) ? cx : 0;
		centerY = (cy != null) ? cy : 0;

		x = centerX + radius * Math.sin(angle + rotationRad);
		y = centerY - radius * Math.cos(angle + rotationRad);

		target = new Array();

		setExtraVars();
		setHitArea();
	}


	public function scale(ratio : Float) {
		scaleX *= ratio;
		scaleY *= ratio;
		radius *= ratio;
	}


	public function moveTo(cx : Float, cy : Float) {
		centerX = cx;
		centerY = cy;
		x = centerX + radius * Math.sin(angle + rotationRad);
		y = centerY - radius * Math.cos(angle + rotationRad);
	}


	public function addWeapon(weapon : Weapon) {
		this.weapon.push(weapon);
	}

	public function removeWeapon(name : String) {
		var i : Int;
		for (i in 0...this.weapon.length) 
			if (this.weapon[i].name == name)
				this.weapon.splice(i, 1);
	}

	public function replaceWeapon(weapon : Weapon) {
		this.weapon.splice(0, this.weapon.length);
		addWeapon(weapon);
	}


	function setExtraVars() : Void {}	// fonction abstraite


	function setHitArea() : Void {
		hitArea = new Sprite();

		//hitArea.graphics.beginFill(0xFF0000, 0.5);
		hitArea.graphics.moveTo(5, 5);
		hitArea.graphics.lineTo(bdWidth - 5, 5);
		hitArea.graphics.lineTo(bdWidth - 5, bdHeight - 5);
		hitArea.graphics.lineTo(5, bdHeight - 5);
		hitArea.graphics.lineTo(5, 5);
		//hitArea.graphics.endFill();

		this.addChild(hitArea);
	}


	public function move() {
		if (exploseOn) 
			exploseRender();

		if (blinkOn)
			blinkRender();

		if (flyBehavior != null)
			flyBehavior.move();
	}


	public function fire() : Array<FlyingObject> {
		var ret : Array<FlyingObject> = new Array();

		if (state == "ok" && Math.random() < fireRate) {
			for (w in weapon) {
				ret = ret.concat(w.fire());
			}
		}

		return ret;
	}


	public function explose(duration : Float) {
		state = "explose";
		exploseOn = true;
		exploseTimer = new Timer(duration);
		exploseTimer.addEventListener(TimerEvent.TIMER, exploseOff);
		exploseTimer.start();
	}


	function exploseRender() {
		alpha *= 0.85;
		scale(1.2);
	}


	function exploseOff(e : TimerEvent) {
		exploseTimer.stop();
		exploseTimer.removeEventListener(TimerEvent.TIMER, exploseOff);
		removeSelf();
	}


	public function blink(duration : Float) {
		blinkOn = true;
		colorAngle = 0;
		colorAngleSpeed = Math.PI * 30 / duration;

		blinkTimer = new Timer(duration);
		blinkTimer.addEventListener(TimerEvent.TIMER, blinkOff);
		blinkTimer.start();
	}


	function blinkRender() {
		colorFx.redOffset = 200 * Math.sin(colorAngle);
		colorFx.greenOffset = 200 * Math.sin(colorAngle);
		colorFx.blueOffset = 200 * Math.sin(colorAngle);
		transform.colorTransform = colorFx;
		colorAngle += colorAngleSpeed;
	}


	function blinkOff(e : TimerEvent) {
		blinkOn = false;
		colorFx.redOffset = 0;
		colorFx.greenOffset = 0;
		colorFx.blueOffset = 0;
		transform.colorTransform = colorFx;

		blinkTimer.stop();
		blinkTimer.removeEventListener(TimerEvent.TIMER, blinkOff);
	}


	public function removeSelf() {
		state = "toRemove";
	}
}

