import flash.display.Loader;
import flash.display.DisplayObject;
import flash.display.GradientType;
import flash.display.MovieClip;
import flash.display.Sprite;
import flash.events.Event;
import flash.events.KeyboardEvent;
import flash.events.MouseEvent;
import flash.events.TimerEvent;
import flash.utils.Timer;
import flash.geom.Point;
import flash.geom.Matrix;
import flash.text.TextField;
import flash.text.TextFormat;
import flash.text.TextFieldAutoSize;
import flash.media.Sound;
import flash.media.SoundChannel;
import flash.net.URLRequest;
import flash.ui.Mouse;



class Main extends MovieClip {
	static var version : String = "1.0.5";

	var currentTime : Float;
#if DEBUG
	var lastCurrentTime : Float;
	var elapsedTime : Float;
	var timeCounter : Int;
#end
	var generateTimer : Timer;

	var keyLeft : Bool;
	var keyRight : Bool;
	var keyUp : Bool;
	var keyDown : Bool;
	var keySpace : Bool;
	var keySpecial : Bool;

	var bg : Sprite; 
	var sky : Sky;
	var space : SpaceContainer;
	var star : Array<Star>;
	var ship : FlyingObject;

	var scenarioManager : ScenarioManager;
	var level : Int;

	var lifeBar : LifeBar;
	var energyBar : FuelBar;
	var energy : Int;
	var score : Int;
	var scoreField : InfoField;
	var bonusLevel : Int;
	var messageField : MessageField;
#if DEBUG
	var debugField : InfoField;
	var debugTimeField : InfoField;
#end

	//var sndGame : Sound;
	//var sndLaser : Sound;
	//var sndExplosion : Sound;
	//var sndCExplosion : SoundChannel;


	static function main() {
		new Main();
	}


	public function new() {
		super();
		flash.Lib.current.addChild(this);

		if (stage == null)
			addEventListener(Event.ADDED_TO_STAGE, onStage);
		else
			launchGame();
	}


	function onStage(e : Event) {
		removeEventListener(Event.ADDED_TO_STAGE, onStage);
		launchGame();
	}


	function launchGame() {
		GContext.globalWidth = stage.stageWidth;
		GContext.globalHeight = stage.stageHeight;
		GContext.stage = stage;

		createObjects();
		createNewGame();

		Mouse.hide();
		addEventListener(Event.ENTER_FRAME, mainLoop);
	}


	function createObjects() {
		// Rectangle bleu pour le fond
		bg = new Sprite();
#if DEBUG
		bg.graphics.beginFill(0x555555);
#else
		bg.graphics.beginFill(0x000000);
#end
		bg.graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
		bg.graphics.endFill();
		addChild(bg);

		// SpaceContainer
		space = new SpaceContainer();
		addChild(space);

		// Ciel
		sky = new Sky();
		sky.y = -sky.height/2;
#if DEBUG
		sky.alpha = 0.1;
#end
		space.addChild(sky);

		// Stars
		star = new Array();
		for (i in 0...50) {
			star[i] = new Star(stage.stageWidth, stage.stageHeight);
			space.addChild(star[i]);
		}

		// Barre de vie 
		lifeBar = new LifeBar(5, stage.stageHeight - 15, 190, 10);
		lifeBar.update(100);
		lifeBar.alpha = 0.7;
		addChild(lifeBar);

		// Le score
		scoreField = new InfoField(stage.stageWidth - 120, 5, 115, 20);
		addChild(scoreField);

		// Barre d'energie
		energyBar = new FuelBar(5, 5, 150, 20, 20);
		energyBar.update(0);
		addChild(energyBar);

		// Informations
#if DEBUG		
		debugField = new InfoField(stage.stageWidth/2 - 40, 5, 80, 20);
		addChild(debugField);

		debugTimeField = new InfoField(stage.stageWidth/2 - 40, 30, 80, 20);
		addChild(debugTimeField);
#end

		// Les messages
		messageField = new MessageField(stage.stageWidth/2, stage.stageHeight/3);
		addChild(messageField);

		// Les sons
		//sndGame = new Sound();
		//sndGame.load(new URLRequest("sound/fondSonore.mp3"));

		//sndLaser = new SoundLaser();
		//sndLaser = new Sound();
		//sndLaser.load(new URLRequest("sound/laserGun2.mp3"));

		//sndExplosion = new SoundExplosion();
		//sndCExplosion = new SoundChannel();
		//sndExplosion = new Sound();
		//sndExplosion.load(new URLRequest("sound/explosion.mp3"));

		// Listes des projectiles (du vaisseau et ennemis)
		GContext.goodProjectile = new Array();
		GContext.badProjectile = new Array();

		// Liste des explosions
		GContext.explosion = new Array();

		// Liste des vaisseaux ennemis
		GContext.ennemy = new Array();

		// Liste des bonus speciaux
		GContext.bonus = new Array();

		// L'activation du generateur de niveaux
		generateTimer = new Timer(1000);
		generateTimer.addEventListener(TimerEvent.TIMER, generateTimerHandler);
		stage.addEventListener("Level.COMPLETED", onLevelUp);
	}


	function createNewGame() {
		score = 0;
		energy = 0;
		level = 0;
		bonusLevel = 0;

		createShip();

		for (ob in GContext.ennemy)
			space.removeChild(ob);
		GContext.ennemy.splice(0, GContext.ennemy.length);

		for (ob in GContext.bonus)
			space.removeChild(ob);
		GContext.bonus.splice(0, GContext.bonus.length);

		for (ob in GContext.goodProjectile)
			space.removeChild(ob);
		GContext.goodProjectile.splice(0, GContext.goodProjectile.length);

		for (ob in GContext.badProjectile)
			space.removeChild(ob);
		GContext.badProjectile.splice(0, GContext.badProjectile.length);

		for (ob in GContext.explosion)
			space.removeChild(ob);
		GContext.explosion.splice(0, GContext.explosion.length);

		// Le generateur de niveaux
		scenarioManager = new ScenarioManager(space, GContext);
		scenarioManager.setScenario(Scenario.level[level]);

		scoreField.update("Score: " + score);
		lifeBar.update(ship.life);
		energyBar.update(energy);

		messageField.show("Level 1\nGo !", 2000);

#if DEBUG
		elapsedTime = 0;
		timeCounter = 0;
		lastCurrentTime = flash.Lib.getTimer();
#end

		// Evenements
		stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
		stage.addEventListener(KeyboardEvent.KEY_UP, onKeyUp);

		generateTimer.start();
	}


	function createShip() {
		// Creation du vaisseau
		ship = new ShipGood(stage.stageWidth / 2, stage.stageHeight - 40, -Math.PI);
		ship.flyBehavior = new FlyBehaviorHorizontal(ship, new OutOfScreenBehaviorClosedBox(ship));
		ship.target = GContext.ennemy;

		ship.gravityEffect = 0;
		ship.easingEffect = 0.75;
		ship.speed = 3;
		ship.life = 100;

		ship.addWeapon(new WeaponUniq("laser", ship, 300));
		ship.fireRate = 1;
		ship.shield = new Shield(ship);
		ship.extraLaserWings = new ExtraLaserWings(ship);

		space.addChild(ship);
		GContext.ship = ship;
	}


	function onKeyDown(e : KeyboardEvent) {
		switch (e.keyCode) {
			case 37: keyLeft = true;
			case 38: keyUp = true;
			case 39: keyRight = true;
			case 40: keyDown = true;
			case 32: keySpace = true;
			case 83: keySpecial = true; // "s"
			default :
		}
	}


	function onKeyUp(e : KeyboardEvent) {
		switch (e.keyCode) {
			case 37: keyLeft = false;
			case 38: keyUp = false;
			case 39: keyRight = false;
			case 40: keyDown = false;
			case 32: keySpace = false;
			case 83: keySpecial = false; // "s"
			default :
		}
	}


	function mainLoop(e : Event) {
		currentTime = flash.Lib.getTimer();
#if DEBUG
		timeCounter++;
		elapsedTime += (currentTime - lastCurrentTime);
		lastCurrentTime = currentTime;
#end

		// Mouvements du vaisseau
		if (ship.state == "ok") {
			if (keyUp) 
				ship.keyUp();

			if (keyDown) 
				ship.keyDown();

			if (keyLeft) 
				ship.keyLeft();

			if (keyRight) 
				ship.keyRight();

		}
		ship.move();

		// Collision avec les ennemis
		for (e in GContext.ennemy) {
			if (e.state == "ok" && ship.state == "ok" 
				&& ship.hitArea.hitTestObject(e.hitArea)) {

					if (ship.shield == null || ship.shield.shieldOn == false) {
						if (bonusLevel > 0) {
							bonusLevel--;
							setWeapon();
						}
						else
							ship.life -= (ship.life < 20) ? ship.life : 20;

						if (ship.life == 0) {
							ship.explose(500);
							gameOver();
						}

						e.life -= 5;
						if (e.life <= 0) {
							ennemyKilled(e);
							scenarioManager.removeItemFrom(e, GContext.ennemy);
						}
						else {
							e.blink(100);
						}
					}

				space.shakeOn(20);
			}
		}

		// Collision avec les Bonus
		for (b in GContext.bonus) {
			if (b.state == "ok" && ship.state == "ok" && ship.hitArea.hitTestObject(b.hitArea)) {
				if (b.name == "bonusred") {
					bonusLevel++;
					messageField.show("New Weapon !", 2000);
					setWeapon();
				}

				else if (b.name == "bonuslife") {
					//ship.life += ( (ship.life <= 60) ? 40 : 100 - ship.life );
					ship.life = 100;
					lifeBar.update(ship.life);
					lifeBar.fxOn(2000);
					messageField.show("Extra life", 2000);
				}

				else if (b.name == "bonusyellow") {
					ship.shield.on(12000);
					//ship.life += ( (ship.life <= 80) ? 20 : 100 - ship.life );
					messageField.show("Shield", 2000);
				}

				//b.removeSelf();
				scenarioManager.removeItemFrom(b, GContext.bonus);
			}
		}

		// Deplacement des projectiles
		for (p in GContext.goodProjectile) {
			p.move();

			for (e in GContext.ennemy) {
				if (p.state == "ok" && e.state == "ok" && p.hitArea.hitTestObject(e.hitArea)) {
					p.removeSelf();

					e.life--;
					if (e.life <= 0) {
						ennemyKilled(e);
						scenarioManager.removeItemFrom(e, GContext.ennemy);
					}
					else {
						e.blink(100);
						new Explosion(space, GContext.explosion, p.centerX, p.centerY, 50, 1);
					}
				}
			}
		}

		for (p in GContext.badProjectile) {
			p.move();

			if (ship.state == "ok" && p.hitArea.hitTestObject(ship.hitArea)) {
				p.removeSelf();

				if (ship.shield == null || ship.shield.shieldOn == false) {
					if (bonusLevel > 0) {
						bonusLevel--;
						setWeapon();
					}
					else
						ship.life -= (ship.life < 10) ? ship.life : 10;

					if (ship.life == 0) {
						ship.explose(500);
						gameOver();
					}

					space.shakeOn(10);
				}

				new Explosion(space, GContext.explosion, p.centerX, p.centerY, 50, 1);
			}
		}

		// Deplacement et tirs des ennemis
		for (e in GContext.ennemy) {
			e.move();
			if (e.state == "ok") {
				var pArray : Array<FlyingObject> = e.fire();
				for (p in pArray) {
					space.addChild(p);
					GContext.badProjectile.push(p);
				}
			}
		}

		// Deplacement des bonus
		for (b in GContext.bonus)
			b.move();

		// Tirs du vaisseau
		if (keySpace) {
			var pArray : Array<FlyingObject> = ship.fire();
			for (p in pArray) {
				space.addChild(p);
				GContext.goodProjectile.push(p);
			}
		}

		// Gestion des explosions
		for (e in GContext.explosion) 
			e.move();

		// Deplacement du ciel et des etoiles 
		sky.move();
		for (s in star) 
			s.move();

		// Tremblements (si shakeOn())
		space.move();

		// ---
		// Suppression des sprites detruits (GC)
		// ---
		for (e in GContext.ennemy)
			if (e.state == "toRemove")
				scenarioManager.removeItemFrom(e, GContext.ennemy);

		for (i in 0...GContext.badProjectile.length)
			if (GContext.badProjectile[i].state == "toRemove") {
				space.removeChild(GContext.badProjectile[i]);
				GContext.badProjectile.splice(i, 1);
			}

		for (i in 0...GContext.goodProjectile.length)
			if (GContext.goodProjectile[i].state == "toRemove") {
				space.removeChild(GContext.goodProjectile[i]);
				GContext.goodProjectile.splice(i, 1);
			}

		for (i in 0...GContext.explosion.length)
			if (GContext.explosion[i].state == "toRemove") {
				space.removeChild(GContext.explosion[i]);
				GContext.explosion.splice(i, 1);
			}

		// Mise a jour du score et de la barre de vie
		scoreField.update("Score: " + score);
		lifeBar.update(ship.life);
		energyBar.update(energy); 

		if (energy >= 100) {
			energy -= 100;
			getBonus();
		}
	}


	function ennemyKilled(e : FlyingObject) {
		e.removeSelf();
		if (e.name == "bigboss" || e.name == "smallboss") {
			new Explosion(space, GContext.explosion, e.centerX, e.centerY, 400, 10);
			score += 20;
			energy += 20;
		}
		else {
			new Explosion(space, GContext.explosion, e.centerX, e.centerY, 200, 3);
			score += 5;
			energy += 5;
		}
		//sndCExplosion.stop();
		//sndCExplosion = sndExplosion.play(0);
	}


	function getBonus() {
		var rand : Int = Std.random(5);

		if (rand < 2)
			scenarioManager.createItem("bonusyellow", "bonus");
		else if (rand < 4) 
			scenarioManager.createItem("bonuslife", "bonus");
		else 
			scenarioManager.createItem("bonusred", "bonus");

	}


	function setWeapon() {
		ship.extraLaserWings.hide();
		switch(bonusLevel) {
			case 0: 
				ship.replaceWeapon(new WeaponUniq("laser", ship, 300));
			case 1:
				ship.replaceWeapon(new WeaponWings("laser", ship, 300));
				ship.extraLaserWings.show();
			case 2:
				ship.replaceWeapon(new WeaponDouble("laser", ship, 300));
			case 3:
				ship.replaceWeapon(new WeaponUniq("laser", ship, 300));
				ship.addWeapon(new WeaponVDouble("laser", ship, 300));
			case 4:
				ship.replaceWeapon(new WeaponUniq("laser", ship, 300));
				ship.addWeapon(new WeaponVDouble("laser", ship, 300));
				ship.addWeapon(new WeaponSides("redlaser", ship, 600));
			case 5:
				ship.replaceWeapon(new WeaponUniq("laser", ship, 300));
				ship.addWeapon(new WeaponWings("laser", ship, 300));
				ship.addWeapon(new WeaponSides("redlaser", ship, 600));
				ship.extraLaserWings.show();
			case 6:
				ship.replaceWeapon(new WeaponDouble("laser", ship, 300));
				ship.addWeapon(new WeaponVDouble("laser", ship, 300));
				ship.addWeapon(new WeaponSides("redlaser", ship, 600));
			case 7:
				ship.replaceWeapon(new WeaponDouble("laser", ship, 300));
				ship.addWeapon(new WeaponVDouble("laser", ship, 300));
				ship.addWeapon(new WeaponSides("redlaser", ship, 600));
				ship.addWeapon(new WeaponGuide("bluecircle", ship, 1200));
		}
	}


	function onLevelUp(e : Event) {
		level++;
		score += level * 100;

		if (Scenario.level[level] != null) {
			bonusLevel++;
			setWeapon();

			scenarioManager.setScenario(Scenario.level[level]);
			messageField.show("Level " + (level + 1), 2000);
		}
		else {
			gameOver();
		}
	}


	function generateTimerHandler(e : TimerEvent) {
		// Generation aleatoire d'ennemis
		scenarioManager.generateItems();
#if DEBUG		
		debugField.update("" + currentTime / 1000);
		debugTimeField.update("" + 1000 * timeCounter / elapsedTime);
		elapsedTime = 0;
		timeCounter = 0;
#end
	}


	function gameOver() {
		Mouse.show();
		messageField.show("Game End\n" + score + " pts !\n Clic to play");
		generateTimer.stop();
		stage.addEventListener(MouseEvent.MOUSE_DOWN, onClick);
	}


	function onClick(e : MouseEvent) {
		stage.removeEventListener(MouseEvent.MOUSE_DOWN, onClick);
		createNewGame();
		Mouse.hide();
	}
}

