import flash.display.DisplayObjectContainer;
import flash.display.MovieClip;
import flash.events.Event;
import flash.events.EventDispatcher;


class ScenarioManager {
	// Stage seront places les aliens
	var stageRef : DisplayObjectContainer;

	// Contexte global du jeu
	var context : Dynamic;

	// Scenario
	var scenario : Array<ScenarioFeedEntry>;
	var entryIdx : Int;

	// Repere de temps
	var timeLine : Int; // dernier "tic" de temps enregistre. En principe, 1 tic = 1 seconde

	// Reference les aliens en jeu (type, %apparition, nombre max, etc.)
	var itemList : Hash<ScenarioEntry>; 


	public function new(stageRef : DisplayObjectContainer, context : Dynamic) {
		this.stageRef = stageRef;
		this.context = context;
		scenario = null;
		itemList = new Hash();
		entryIdx = 0;
	}


	public function setScenario(scenario : Array<ScenarioFeedEntry>) {
		var itemEntry : ScenarioEntry;
		var key : String;

		timeLine = 0;
		entryIdx = 0;

		// Reinitialise les entrees
		for (key in itemList.keys()) {
			itemEntry = itemList.get(key);

			itemEntry.percent = 0;
			itemEntry.max = 0;
			itemEntry.uniq = false;
			itemEntry.pattern = '';
			itemEntry.signal = null;
			itemEntry.gen = 0;

			itemList.set(itemEntry.type, itemEntry);
		}
		this.scenario = scenario;
	}


	public function removeItemFrom(flyObj : FlyingObject, flyArray : Array<FlyingObject>) : Bool {
		var itemEntry : ScenarioEntry;
		var i : Int;

		for (i in 0...flyArray.length) {
			if (flyArray[i] == flyObj) {
				itemEntry = itemList.get(flyObj.name);
				if (itemEntry != null) {
					itemEntry.count--;
					itemList.set(flyObj.name, itemEntry);

					if (itemEntry.signal != null && itemEntry.count == 0 && itemEntry.uniq && itemEntry.gen >= itemEntry.max) 
						context.stage.dispatchEvent(new Event(itemEntry.signal));
				}
				//trace(itemEntry.type + " " + itemEntry.percent + "% n:" + itemEntry.count + " max:" + itemEntry.max + " (" + itemEntry.uniq + ") gen:" + itemEntry.gen);

				stageRef.removeChild(flyObj);
				flyArray.splice(i, 1);
				return true;
			}
		}

		trace("ERROR: OBJECT NOT FOUND: " + flyObj.name);
		return false;
	}


	// Cree une vague d'aliens
	public function createItem(type : String, ?pattern : String) {
		var x : Float;
		var y : Float;
		var radian : Float;
		var flyingObject : FlyingObject;

		var itemEntry : ScenarioEntry = itemList.get(type);
		if (itemEntry == null) {
			itemEntry = new ScenarioEntry(type, 0, 0, 0, false, null, null);
			itemList.set(type, itemEntry);
		}

		switch (pattern) {
			case "column4":
				itemEntry.count += 4;
				itemEntry.gen += 4;
				itemList.set(type, itemEntry);

				flyingObject = FlyerFactory.createFlyer(type, 0, 0, 0);
				x = Std.random(cast(context.globalWidth - 2*flyingObject.bdWidth, Int)) + flyingObject.bdWidth/2;
				y = -flyingObject.bdHeight;
				flyingObject.moveTo(x, y);

				flyingObject.target.push(context.ship); 
				context.ennemy.push(flyingObject); 
				stageRef.addChild(flyingObject);

				for (i in 1...4) {
					flyingObject = FlyerFactory.createFlyer(type, x, y - flyingObject.bdHeight * i, 0);

					flyingObject.target.push(context.ship); 
					context.ennemy.push(flyingObject); 
					stageRef.addChild(flyingObject);
				}

			case "line4":
				itemEntry.count += 4;
				itemEntry.gen += 4;
				itemList.set(type, itemEntry);

				flyingObject = FlyerFactory.createFlyer(type, 0, 0, 0);
				var width = flyingObject.bdWidth * 4.25;
				x = Std.random(cast(context.globalWidth - width, Int)) + flyingObject.bdWidth/2;
				y = -flyingObject.bdHeight;
				flyingObject.moveTo(x, y);

				flyingObject.target.push(context.ship); 
				context.ennemy.push(flyingObject); 
				stageRef.addChild(flyingObject);

				for (i in 1...4) {
					flyingObject = FlyerFactory.createFlyer(type, x + i * 1.25 * flyingObject.bdWidth, y, 0);
					flyingObject.target.push(context.ship); 
					context.ennemy.push(flyingObject); 
					stageRef.addChild(flyingObject);
				}

			case "lineFull":
				flyingObject = FlyerFactory.createFlyer(type, 0, 0, 0);
				var n : Int = cast (context.globalWidth / (1.25 * flyingObject.bdWidth), Int);
				x = flyingObject.bdWidth/2;
				y = -flyingObject.bdHeight;
				flyingObject.moveTo(x, y);

				flyingObject.target.push(context.ship); 
				context.ennemy.push(flyingObject); 
				stageRef.addChild(flyingObject);

				for (i in 1...n) {
					flyingObject = FlyerFactory.createFlyer(type, x + i * 1.25 * flyingObject.bdWidth, y, 0);
					flyingObject.target.push(context.ship); 
					context.ennemy.push(flyingObject); 
					stageRef.addChild(flyingObject);
				}

				itemEntry.count += n;
				itemEntry.gen += n;
				itemList.set(type, itemEntry);

			case "rand2":
				itemEntry.count += 2;
				itemEntry.gen += 2;
				itemList.set(type, itemEntry);

				flyingObject = FlyerFactory.createFlyer(type, 0, 0, 0);
				var n : Int = cast (context.globalWidth / flyingObject.bdWidth, Int);
				x = flyingObject.bdWidth/2 + Std.random(n) * flyingObject.bdWidth;
				y = -flyingObject.bdHeight;
				flyingObject.moveTo(x, y);

				flyingObject.target.push(context.ship); 
				context.ennemy.push(flyingObject); 
				stageRef.addChild(flyingObject);

				var x2 = x;
				while (x2 == x) {
					x2 = flyingObject.bdWidth/2 + Std.random(n) * flyingObject.bdWidth;
				}
				flyingObject = FlyerFactory.createFlyer(type, x2, y, 0);

				flyingObject.target.push(context.ship); 
				context.ennemy.push(flyingObject); 
				stageRef.addChild(flyingObject);

			
			case "middle1":
				itemEntry.count++;
				itemEntry.gen++;
				itemList.set(type, itemEntry);

				flyingObject = FlyerFactory.createFlyer(type, 0, 0, 0);
				x = context.globalWidth / 2;
				y = -flyingObject.bdHeight;
				flyingObject.moveTo(x, y);

				flyingObject.target.push(context.ship); 
				context.ennemy.push(flyingObject); 
				stageRef.addChild(flyingObject);

			case "bonus":
				itemEntry.count++;
				itemEntry.gen++;
				itemList.set(type, itemEntry);

				// Creation et placement
				flyingObject = FlyerFactory.createFlyer(type, 0, 0, 0);
				x = Std.random(cast(context.globalWidth - 2*flyingObject.bdWidth, Int)) + flyingObject.bdWidth/2;
				y = -flyingObject.bdHeight;
				flyingObject.moveTo(x, y);

				context.bonus.push(flyingObject); 
				stageRef.addChild(flyingObject);

			default:
				itemEntry.count++;
				itemEntry.gen++;
				itemList.set(type, itemEntry);

				// Creation et placement
				flyingObject = FlyerFactory.createFlyer(type, 0, 0, 0);
				x = Std.random(cast(context.globalWidth - 2*flyingObject.bdWidth, Int)) + flyingObject.bdWidth/2;
				y = -flyingObject.bdHeight;
				flyingObject.moveTo(x, y);

				flyingObject.target.push(context.ship); 
				context.ennemy.push(flyingObject); 
				stageRef.addChild(flyingObject);
		}

	}


	public function generateItems() {
		var itemEntry : ScenarioEntry;
		var scenarioEntry : ScenarioFeedEntry;
		var p : Int; 
		var flyer : String;
		var value : Int;

		timeLine++;

		//
		// Les entrees du scenario sont lues une apres l'autre.
		// La lecture bloque si l'entree en cours n'est pas encore valide.
		// Une fois lue, la ligne est supprimee.
		//
		while (entryIdx < scenario.length && scenario[entryIdx].timeLine <= timeLine) {
			scenarioEntry = scenario[entryIdx];
			entryIdx++;

			//
			// L'entree de la table des aliens est mise a jour
			//
			itemEntry = itemList.get(scenarioEntry.type);
			if (itemEntry != null) {
				itemEntry.type = scenarioEntry.type;
				itemEntry.percent = scenarioEntry.percent;
				itemEntry.max = Math.abs(scenarioEntry.max);
				itemEntry.uniq = (scenarioEntry.max < 0);
				itemEntry.pattern = scenarioEntry.pattern;
				itemEntry.signal = scenarioEntry.signal;
				itemEntry.gen = 0;
			}
			else {
				itemEntry = new ScenarioEntry(
					scenarioEntry.type,
					scenarioEntry.percent,
					0,
					Math.abs(scenarioEntry.max),
					(scenarioEntry.max < 0),
					scenarioEntry.pattern,
					scenarioEntry.signal 
				);
			}

			itemList.set(itemEntry.type, itemEntry);
		}

		// Chaque occurence est lue et une nouvelle vague d'aliens est eventuellement creee
		for (flyer in itemList.keys()) {
			itemEntry = itemList.get(flyer);

			if ( (itemEntry.uniq && itemEntry.gen < itemEntry.max) ||
			     (!itemEntry.uniq && itemEntry.count < itemEntry.max) ) {
					if (Math.round(Std.random(100)) <= itemEntry.percent) {
						createItem(flyer, itemEntry.pattern);
						//trace(itemEntry.type + " " + itemEntry.percent + "% n:" + itemEntry.count + " max:" + itemEntry.max + " (" + itemEntry.uniq + ") gen:" + itemEntry.gen);
					}
			}
		}
	}
}

